      * Command to send a signal to a process.  This is different
      * from the QShell 'kill' command in that it can send the
      * signal using the OS/400 job identity.
      *                                Scott Klement, August 14, 2007
      *----------------------------------------------------------------
      *
      * Before compiling:
      *    - upload signal_h.rpgle --> your-lib/QRPGLESRC mbr: SIGNAL_H
      *    - upload killr4.rpgle   --> your-lib/QRPGLESRC mbr: KILLR4
      *    - upload kill.pnlgrp    --> your-lib/QPNLSRC   mbr: KILL
      *    - upload kill.cmd       --> your-lib/QCMDSRC   mbr: KILL
      *
      * To compile:
      *>   CRTBNDRPG KILLR4 SRCFILE(QRPGLESRC) DBGVIEW(*LIST)
      *>   CRTPNLGRP KILL   SRCFILE(QPNLSRC)
      *>   CRTCMD    KILL   SRCFILE(QCMDSRC) -
      *>                    PGM(KILLR4) HLPPNLGRP(KILL) HLPID(KILL)
      *>   CRTCMD    SNDSIG SRCFILE(QCMDSRC) SRCMBR(KILL) -
      *>                    PGM(KILLR4) HLPPNLGRP(KILL) HLPID(KILL)
      *
      * For info on how to call this, see the help in the KILL
      * command.
      *
     /*-                                                                            +
      * Copyright (c) 2009 Scott C. Klement                                         +
      * All rights reserved.                                                        +
      *                                                                             +
      * Redistribution and use in source and binary forms, with or without          +
      * modification, are permitted provided that the following conditions          +
      * are met:                                                                    +
      * 1. Redistributions of source code must retain the above copyright           +
      *    notice, this list of conditions and the following disclaimer.            +
      * 2. Redistributions in binary form must reproduce the above copyright        +
      *    notice, this list of conditions and the following disclaimer in the      +
      *    documentation and/or other materials provided with the distribution.     +
      *                                                                             +
      * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ''AS IS'' AND      +
      * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE       +
      * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE  +
      * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE     +
      * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL  +
      * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS     +
      * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       +
      * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT  +
      * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY   +
      * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF      +
      * SUCH DAMAGE.                                                                +
      *                                                                             +
      */                                                                            +
     H DFTACTGRP(*NO) BNDDIR('QC2LE') OPTION(*SRCSTMT)
     H COPYRIGHT('Copyright (C) 2009 Scott C. Klement.  See Source -
     H for license information.')

     D KILLR4          PR                  ExtPgm('KILLR4')
     D   QualJob                     26a   const
     D   SigNo                       10i 0 const
     D KILLR4          PI
     D   QualJob                     26a   const
     D   SigNo                       10i 0 const

      /copy signal_h

     D get_errno       PR              *   ExtProc('__errno')
     D errno           s             10I 0 based(p_errno)

     D QUSRJOBI        PR                  ExtPgm('QUSRJOBI')
     D   rcvvar                   32767A   options(*varsize)
     D   rcvvarlen                   10i 0 const
     D   format                       8a   const
     D   qualJob                     26a   const
     D   intjobid                    16a   const
     D   ErrorCode                 8000A   options(*varsize:*nopass)

     D JOBI0800        ds                  qualified
     D   BytesReturn                 10i 0
     D   BytesAvail                  10i 0
     D   siginit                     10i 0 overlay(JOBI0800:65)
     D   pid                         10i 0 overlay(JOBI0800:93)

     D ErrorNull       DS                  qualified
     D   BytesProv                   10I 0 inz(0)
     D   BytesAvail                  10I 0 inz(0)

     D QMHRSNEM        PR                  ExtPgm('QMHRSNEM')
     D   MsgKey                       4A   const
     D   ErrorCode                 8000A   options(*varsize)
     D   ToEntry                           likeds(RSNM0100)
     D                                     options(*nopass)
     D   ToEntLen                    10i 0 const options(*nopass)
     D   ToEntFmt                     8a   const options(*nopass)
     D   FromAddr                      *   const options(*nopass)
     D   FromCount                   10i 0 const options(*nopass)

     D RSNM0100        ds                  qualified
     D   counter                     10i 0 inz(1)
     D   module                      10a   inz('*NONE')
     D   pgm                         10a   inz('*NONE')
     D   entrylen                    10i 0 inz(7)
     D   entry                       10a   inz('*PGMBDY')

     D ReportError     PR

      /free

          callp(e) QUSRJOBI( JOBI0800
                           : %size(JOBI0800)
                           : 'JOBI0800'
                           : QualJob
                           : *blanks
                           : ErrorNull );
          if %error;
             QMHRSNEM( *blanks
                     : ErrorNull
                     : RSNM0100
                     : %len(RSNM0100)
                     : 'RSNM0100'
                     : *null
                     : 0 );
             return;
          endif;

          // siginit=0 means the process isn't enabled for
          // signals.  Darn.

          if JOBI0800.siginit = 0;
             p_errno = get_errno();
             errno = 3516;
             ReportError();
             return;
          endif;

          if (kill(JOBI0800.pid: signo) = -1);
             ReportError();
          endif;

          *inlr = *on;

      /end-free


      *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      * ReportError():  Send an escape message explaining any errors
      *                 that occurred.
      *
      *  This function requires binding directory QC2LE in order
      *  to access the __errno() function.
      *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     P ReportError     B
     D ReportError     PI

     D QMHSNDPM        PR                  ExtPgm('QMHSNDPM')
     D   MessageID                    7A   Const
     D   QualMsgF                    20A   Const
     D   MsgData                  32767A   Const options(*varsize)
     D   MsgDtaLen                   10I 0 Const
     D   MsgType                     10A   Const
     D   CallStkEnt                  10A   Const
     D   CallStkCnt                  10I 0 Const
     D   MessageKey                   4A
     D   ErrorCode                32767A   options(*varsize)

     D MsgKey          S              4A
     D MsgID           s              7A

      /free

         p_errno = get_errno();
         MsgID = 'CPE' + %editc(%dec(errno:4:0): 'X');

         QMHSNDPM( MsgID
                 : 'QCPFMSG   *LIBL'
                 : ' '
                 : 0
                 : '*ESCAPE'
                 : '*PGMBDY'
                 : 1
                 : MsgKey
                 : ErrorNull );

      /end-free
     P                 E
